###############################################################################
#  common.py: Module with common functions.
#  Copyright (C) 2010 Brno University of Technology, ANT @ FIT
#  Author(s): Vaclav Bartos <xbarto11@stud.fit.vutbr.cz>
###############################################################################
#
#  LICENSE TERMS
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#  3. All advertising materials mentioning features or use of this software
#     or firmware must display the following acknowledgement:
#
#       This product includes software developed by the University of
#       Technology, Faculty of Information Technology, Brno and its
#       contributors.
#
#  4. Neither the name of the Company nor the names of its contributors
#     may be used to endorse or promote products derived from this
#     software without specific prior written permission.
#
#  This software or firmware is provided ``as is'', and any express or implied
#  warranties, including, but not limited to, the implied warranties of
#  merchantability and fitness for a particular purpose are disclaimed.
#  In no event shall the company or contributors be liable for any
#  direct, indirect, incidental, special, exemplary, or consequential
#  damages (including, but not limited to, procurement of substitute
#  goods or services; loss of use, data, or profits; or business
#  interruption) however caused and on any theory of liability, whether
#  in contract, strict liability, or tort (including negligence or
#  otherwise) arising in any way out of the use of this software, even
#  if advised of the possibility of such damage.
#
#  $Id: common.py 1662 2012-01-29 16:42:11Z ipus $

"""
Module with some common functions. Currently, only function for extracting
fields from packet header is implemented.
"""

from netbench.common.headerfields import *

# -----------------------------------------------------------------------------
# Exception types

# Used by classification algorithm when they can't process given packet
class BadPacketError(Exception):
    pass

# Used by extract_field function in this module
class FieldNotFoundError(Exception):
    pass
class UnknownFieldError(Exception):
    pass


# -----------------------------------------------------------------------------
# Extraction of fields from packet header

def extract_field(header, field_name):
    """
    Extracts value of specified filed from given packet header.
    If the name of field is unknown, an UnknownFieldError exception is raised.
    If the field is not present in packet header, a FieldNotFoundError 
    exception is raised.
    
    header - Instance of PacketHeader, in which the field will be looked for.
    
    field_name - string, name of filed to extract. Possible values:
                   srcipv4
                   dstipv4
                   src_port
                   dst_port
                   protocol
                   tcpflags
    """
    if not table.has_key(field_name):
        raise UnknownFieldError(field_name)
    
    for field in header.iterfields():
        if isinstance(field, table[field_name]):
            return field
    
    raise FieldNotFoundError(field_name)

"""
Table mapping string field names to corresponding types.
"""
table = {
    "srcipv4":  IP4_SrcAddress,
    "dstipv4":  IP4_DstAddress,
    "src_port": SrcPort,
    "dst_port": DstPort,
    "protocol": IP4_Protocol,
    "tcpflags": TCP_Flags
}

dstab = {
    "srcipv4":  32,
    "dstipv4":  32,
    "src_port": 16,
    "dst_port": 16,
    "protocol": 8,
    "tcpflags": 8
}
