#!/usr/bin/env python
###############################################################################
#  range.py: IP/port range class
#  Copyright (C) 2012 Brno University of Technology, ANT @ FIT
#  Author(s): Martin Foukal <xfouka00@stud.fit.vutbr.cz>
###############################################################################
#
#  LICENSE TERMS
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#  3. All advertising materials mentioning features or use of this software
#     or firmware must display the following acknowledgement:
#
#       This product includes software developed by the University of
#       Technology, Faculty of Information Technology, Brno and its
#       contributors.
#
#  4. Neither the name of the Company nor the names of its contributors
#     may be used to endorse or promote products derived from this
#     software without specific prior written permission.
#
#  This software or firmware is provided ``as is'', and any express or implied
#  warranties, including, but not limited to, the implied warranties of
#  merchantability and fitness for a particular purpose are disclaimed.
#  In no event shall the company or contributors be liable for any
#  direct, indirect, incidental, special, exemplary, or consequential
#  damages (including, but not limited to, procurement of substitute
#  goods or services; loss of use, data, or profits; or business
#  interruption) however caused and on any theory of liability, whether
#  in contract, strict liability, or tort (including negligence or
#  otherwise) arising in any way out of the use of this software, even
#  if advised of the possibility of such damage.
#
#  $Id: $


class Range(object):
    '''
    Represents IP/Port in range format
    '''


    def __init__(self,
                 start_value,
                 end_value,
                 domain_size,                 
                 ):
        
        """
        Constructor        
        
        domain_size: Number of bits of value and mask. 32 for IPv4, 16 for port, ...
        begin:    First number of range        
        end:      Last number of range        
        """
        
        self.domain_size = domain_size
        self.begin = start_value
        self.end = end_value
        
        if self.begin > self.end:
            raise ValueError("Start value cannot be greater than end value")
        
        if self.begin < 0 or self.end >= (1 << domain_size):
            raise ValueError("Values are out of bounds (%d,%d")%(0, (1 << domain_size) - 1) 
        
            
    def covers(self, interval):
        """
        Return True if range fully covers another range(interval).
        """
        return (self.begin <= interval.begin and self.end >= interval.end)
        
        
    def display(self,format=""):        
        print self.__str__(format)
        
                                             
    def __str__(self,format=""):
        """ 
        Formats:
            
        "ip"  e.x "192.168.16.1:192.168.16.255"
        "num" e.x "50000:55000"
        """
        
        if format == "ip" and self.domain_size == 32:            
            s = self.begin
            e = self.end            
            return "%d.%d.%d.%d:%d.%d.%d.%d"%((s>>24)&0xff, (s>>16)&0xff,(s>>8)&0xff,
                                      s&0xff, (e>>24)&0xff, (e>>16)&0xff,(e>>8)&0xff,
                                      e&0xff)
            
        else:            
            return str(self.begin) + ":" + str(self.end)            
                        
                
            
            
            
            
            
        
        
