#!/usr/bin/env python
###############################################################################
#  range_bv.py: IP/port range with bitvector
#  Copyright (C) 2012 Brno University of Technology, ANT @ FIT
#  Author(s): Martin Foukal <xfouka00@stud.fit.vutbr.cz>
###############################################################################
#
#  LICENSE TERMS
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#  3. All advertising materials mentioning features or use of this software
#     or firmware must display the following acknowledgement:
#
#       This product includes software developed by the University of
#       Technology, Faculty of Information Technology, Brno and its
#       contributors.
#
#  4. Neither the name of the Company nor the names of its contributors
#     may be used to endorse or promote products derived from this
#     software without specific prior written permission.
#
#  This software or firmware is provided ``as is'', and any express or implied
#  warranties, including, but not limited to, the implied warranties of
#  merchantability and fitness for a particular purpose are disclaimed.
#  In no event shall the company or contributors be liable for any
#  direct, indirect, incidental, special, exemplary, or consequential
#  damages (including, but not limited to, procurement of substitute
#  goods or services; loss of use, data, or profits; or business
#  interruption) however caused and on any theory of liability, whether
#  in contract, strict liability, or tort (including negligence or
#  otherwise) arising in any way out of the use of this software, even
#  if advised of the possibility of such damage.
#
#  $Id: $

from netbench.classification.range import *

class RangeBV(Range):
    '''
    Range which has bitvector (list, its length is equal of number of rules)
    It's subclass of Range
    '''


    def __init__(self,
                 start_value,
                 end_value,
                 domain_size,                 
                 ):
        
        """
        Constructor        
        
        begin:       First number of range        
        end:         Last number of range        
        domain_size: 32 for IPv4, 16 for port, 8 for protocol        
        """
        
        self.bitvector = []  
        
        """
        Pointer indicating changed bits from previous interval.
        There are at most l-1 intervals between ranges for which bitvectors 
        have been stored.
        Dictionary {index_in_bitvector:int,bit_value:int}
        """
        self.pointers = {}      
            
        Range.__init__(self,
                       start_value,
                       end_value,
                       domain_size
                       )
        
    def has_bitvector(self):
        return self.bitvector != []
    
    def has_pointers(self):
        return self.pointers != {}
        
