import unittest
import traffic
import packet
import packetheader

class BasicTests(unittest.TestCase):
    """Basics tests of the common part of the ANT library."""

    def testTraffic(self):
        """Tests if it is possible to create object of the Traffic class"""
        failed = 0
        try:
            traffic.Traffic()
        except Exception:
            failed = 1
        self.assert_(failed == 0)

    def testnonexistentFile(self):
        """Tries to open nonexisting file without .pcap suffix"""
        failed = 0
        t = traffic.Traffic()
        try:
            Data = t.set_file("nesmysl")
        except Exception:
            failed = 1
        self.assert_(failed ==0 and Data == 0)

    def testnonexistentFile1(self):
        """Tries to open nonexisting file with .pcap suffix."""
        failed = 0
        Data = -1
        t = traffic.Traffic()
        try:
            Data = t.set_file("nesmysl.pcap")
        except Exception:
            failed = 1
        self.assert_(failed ==0 and Data == 0)
    
    def testFile(self):
        """Tries to open existing pcap file"""
        failed = 0
        Data = -1
        t = traffic.Traffic()
        try:
            Data = t.set_file("test.pcap")
        except Exception:
            failed = 1
        self.assert_(failed ==0 and Data == 1)

class AdvancedTests(unittest.TestCase):
    """Contains more advanced tests of the common part of the ANT library"""

    def setUp(self):
        """Sets up the environment for the tests. Packet are loaded from the 
           test.pcap file into the object of self.traffic and packet object is 
           created."""
        self.traffic = traffic.Traffic()
        if not self.traffic.set_file("test.pcap"):
            Raise(Exception)
        self.pckt = packet.Packet()

    def testCount(self):
        """Counts the number of the packets in the file. In test.pcap there 
           should be 306 packets."""
        i=0
        while self.traffic.next_packet(self.pckt):
            i = i+1
        self.assert_(i == 306)
    def testUDP(self):
        """Tests if the netbench parse UDP packet correctly. Test that the values for the first UDP packet are the same as in wireshark"""
        while 1:
           pckt = packet.Packet()
           if not self.traffic.next_packet(pckt): 
               break
           pckt.parse()
           hdr = pckt.get_packet_header()
           
           if('ipv4' in hdr.get_headers_type()):
               protokol = hdr.get_header('ipv4').get_field('protocol')
               if protokol == 17: 
                   break
        self.assert_(hdr.get_header('udp').get_field('src_port')==137)
        self.assert_(hdr.get_header('udp').get_field('dst_port')==137)
        self.assert_(hdr.get_header('udp').get_field('udp_len')==58)
        self.assert_(hdr.get_header('udp').get_field('udp_checksum')==0x5c05)

if __name__ == '__main__':
    #Creates test suite from each test class by loading all functions starting with test.
    suite_A = unittest.TestLoader().loadTestsFromTestCase(AdvancedTests)
    suite = unittest.TestLoader().loadTestsFromTestCase(BasicTests)
    #Adds suite_A into the suite. 
    suite.addTest(suite_A)
    #Runs all test in the test suite called suite. Verbosity=2 means, that the description will be printed for each test.
    unittest.TextTestRunner(verbosity=2).run(suite)

