###############################################################################
#  fields.py: Module with classes representing header fields
#  Copyright (C) 2010 Brno University of Technology, ANT @ FIT
#  Author(s): Vaclav Bartos <xbarto11@stud.fit.vutbr.cz>
###############################################################################
#
#  LICENSE TERMS
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#  3. All advertising materials mentioning features or use of this software
#     or firmware must display the following acknowledgement:
#
#       This product includes software developed by the University of
#       Technology, Faculty of Information Technology, Brno and its
#       contributors.
#
#  4. Neither the name of the Company nor the names of its contributors
#     may be used to endorse or promote products derived from this
#     software without specific prior written permission.
#
#  This software or firmware is provided ``as is'', and any express or implied
#  warranties, including, but not limited to, the implied warranties of
#  merchantability and fitness for a particular purpose are disclaimed.
#  In no event shall the company or contributors be liable for any
#  direct, indirect, incidental, special, exemplary, or consequential
#  damages (including, but not limited to, procurement of substitute
#  goods or services; loss of use, data, or profits; or business
#  interruption) however caused and on any theory of liability, whether
#  in contract, strict liability, or tort (including negligence or
#  otherwise) arising in any way out of the use of this software, even
#  if advised of the possibility of such damage.
#
#  $Id: headerfields.py 1565 2012-01-02 08:54:34Z izadnik $


# Base class for all header fields
class HeaderField(long):
    bits = None
    def __repr__(self):
        return "<%s %s>" % (self.__class__.__name__, self.__str__())
    def __len__(self):
        return self.bits


# -- Ethernet fields --------------------------------------

class Eth_Mac(HeaderField):
    bits = 48
    def __str__(self):
        return "%02x:%02x:%02x:%02x:%02x:%02x"%(
               (self >> 40) & 0xff,
               (self >> 32) & 0xff,
               (self >> 24) & 0xff,
               (self >> 16) & 0xff,
               (self >>  8) & 0xff,
               self & 0xff )

class Eth_SrcMac(Eth_Mac):
    pass
class Eth_DstMac(Eth_Mac):
    pass

class Eth_TypeLength(HeaderField):
    bits = 16


# -- IPv4 fields ------------------------------------------

class IP4_Version(HeaderField):
    bits = 4

class IP4_HeaderLen(HeaderField):
    bits = 4

class IP4_ToS(HeaderField):
    bits = 8

class IP4_TotalLen(HeaderField):
    bits = 16

class IP4_ID(HeaderField):
    bits = 16

class IP4_Flags(HeaderField):
    bits = 3

class IP4_FragmentOffset(HeaderField):
    bits = 13

class IP4_TTL(HeaderField):
    bits = 8

class IP4_Protocol(HeaderField):
    bits = 8
    names = {# Names of some protocols, add more if needed 
             1:  "ICMP",
             2:  "IGMP",
             4:  "IP",
             6:  "TCP",
             17: "UDP",
             27: "RDP",
             41: "IPv6",
             51: "AH",
             89: "OSPF",
            }
    def toStr(self):
        if IP4_Protocol.names.has_key(self):
            return IP4_Protocol.names[self]
        else:
            return str(int(self))

class IP4_Checksum(HeaderField):
    bits = 16

class IP4_Address(HeaderField):
    bits = 32
    def __str__(self):
        return str((self >> 24) & 0xff) + "." + \
               str((self >> 16) & 0xff) + "." + \
               str((self >>  8) & 0xff) + "." + \
               str(self & 0xff)
class IP4_SrcAddress(IP4_Address):
    pass
class IP4_DstAddress(IP4_Address):
    pass

class IP4_Options(HeaderField):
    bits = None # Length of IPv4 options is variable


# -- TCP and UDP Ports ------------------------------------

class Port(HeaderField):
    bits = 16

class SrcPort(Port):
    pass
class UDP_SrcPort(SrcPort):
    pass
class TCP_SrcPort(SrcPort):
    pass

class DstPort(Port):
    bits = 16
class UDP_DstPort(DstPort):
    pass
class TCP_DstPort(DstPort):
    pass

# -- Other TCP fields ------------------------------------- 

class TCP_DataOffset(HeaderField):
    bits = 4

class TCP_Flags(HeaderField):
    bits = 8

class TCP_Window(HeaderField):
    bits = 16 


# -- Other UDP fields ------------------------------------- 

class UDP_Length(HeaderField):
    bits = 16

class UDP_Checksum(HeaderField):
    bits = 16



# -- 802.1 Vitrual LAN fields --------------------------------------

class VLAN_Type(HeaderField):
    bits = 16
    def __str__(self):
        return "%04x"%self



