###############################################################################
#  test_sst.py: Test module for BSI LPM method
#  Copyright (C) 2010 Brno University of Technology, ANT @ FIT
#  Author(s): Frantisek Sedlar <xsedla76@stud.fit.vutbr.cz>
###############################################################################
#
#  LICENSE TERMS
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#  3. All advertising materials mentioning features or use of this software
#     or firmware must display the following acknowledgement:
#
#       This product includes software developed by the University of
#       Technology, Faculty of Information Technology, Brno and its
#       contributors.
#
#  4. Neither the name of the Company nor the names of its contributors
#     may be used to endorse or promote products derived from this
#     software without specific prior written permission.
#
#  This software or firmware is provided ``as is'', and any express or implied
#  warranties, including, but not limited to, the implied warranties of
#  merchantability and fitness for a particular purpose are disclaimed.
#  In no event shall the company or contributors be liable for any
#  direct, indirect, incidental, special, exemplary, or consequential
#  damages (including, but not limited to, procurement of substitute
#  goods or services; loss of use, data, or profits; or business
#  interruption) however caused and on any theory of liability, whether
#  in contract, strict liability, or tort (including negligence or
#  otherwise) arising in any way out of the use of this software, even
#  if advised of the possibility of such damage.
#
#  $Id$

"""
Test module for BSI LPM method
"""

import sys, os
import netbench.lpm.parsers.genericparser as genericparser
import netbench.lpm.algorithms.bsi as bsi
from netbench.classification import maskedint

path = "../prefixes/test/"
prefixsets = (os.listdir(path))

outfile = "test_bsi.out"

# compute and print memory demands and stats of currently loaded prefixset
def compute():
    file.write("\n*************** Filename: " + filename + " ***************\n")
    file.write("Count of successful parsed prefixes: " + str(parser.get_parsed_prefixes()))
    #file.write("\nCount of invalid prefixes: " + str(parser.get_invalid_prefixes()))
    #file.write("\nHistogram:\n\t")
    # get histogram of currently loaded prefixset
    #histogram = prefixset.get_histogram()
    #for i in histogram:
    #    file.write(str(i) + " ")

    file.write("\nMean length of prefixes: " + str(prefixset.get_mean_length()))
    
    file.write("\n --- BSI ---")
    file.write("\n" + 40*"-")
    my_bsi = bsi.BSI()
    my_bsi.load_prefixset(prefixset)
    report = my_bsi.report_memory()
    file.write("\nPrefixes: " + str(report['prefixes']))
    file.write("\nLines of table: " + str(report['size(lines) of table']))
    file.write("\nMemory demands [Bytes]\n")
    file.write("32-bit pointer: " + str (report['size(lines) of table']*(16+4)))
    # compute minimal possible pointer size
    prefix_ptr = maskedint.log2(report['prefixes'])
    file.write("\nSmallest possible: " + str(report['size(lines) of table']*(128+prefix_ptr)/8.0))
    file.write("\n" + 80*"*" + "\n")

# try to open a file
try:
    file = open(outfile, 'w')
except IOError:
    print "Cannot open file for writing results"
    exit()

       
print "COMPUTING..."
file.write("\n\n\n==================================================\n")
file.write("----------=======  TEST PREFIXES  =======---------\n")
file.write("==================================================\n")

for prefixset in prefixsets:
    filename = path + prefixset
    parser = genericparser.GenericParser()
    prefixset = parser.get_prefixes()
    if (parser.load_file(filename)):
        print "test file:", filename
        compute()
    else:
        print "Cannot load file",filename
print "DONE.."
print "Results are in file", outfile
file.close()
