###############################################################################
#  test_v6_lctrie.py: Test module for LC-Trie LPM method
#  Copyright (C) 2011 Brno University of Technology, ANT @ FIT
#  Author(s): Frantisek Sedlar <xsedla76@stud.fit.vutbr.cz>
###############################################################################
#
#  LICENSE TERMS
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#  3. All advertising materials mentioning features or use of this software
#     or firmware must display the following acknowledgement:
#
#       This product includes software developed by the University of
#       Technology, Faculty of Information Technology, Brno and its
#       contributors.
#
#  4. Neither the name of the Company nor the names of its contributors
#     may be used to endorse or promote products derived from this
#     software without specific prior written permission.
#
#  This software or firmware is provided ``as is'', and any express or implied
#  warranties, including, but not limited to, the implied warranties of
#  merchantability and fitness for a particular purpose are disclaimed.
#  In no event shall the company or contributors be liable for any
#  direct, indirect, incidental, special, exemplary, or consequential
#  damages (including, but not limited to, procurement of substitute
#  goods or services; loss of use, data, or profits; or business
#  interruption) however caused and on any theory of liability, whether
#  in contract, strict liability, or tort (including negligence or
#  otherwise) arising in any way out of the use of this software, even
#  if advised of the possibility of such damage.
#
#  $Id$

"""
Test module for LC-Trie LPM method
"""

import sys, os
import netbench.lpm.parsers.genericparser as genericparser
import netbench.lpm.algorithms.lctrie as lctrie
from netbench.classification import maskedint

path = "../prefixes/test/"
prefixsets = (os.listdir(path))

outfile = "test_lctrie.out"

# compute and print memory demands and stats of currently loaded prefixset
def compute():
    file.write("\n*************** Filename: " + filename + " ***************\n")
    file.write("Count of successful parsed prefixes: " + str(parser.get_parsed_prefixes()))
    #file.write("\nCount of invalid prefixes: " + str(parser.get_invalid_prefixes()))
    file.write("\nHistogram:\n\t")
    # get histogram of currently loaded prefixset
    histogram = prefixset.get_histogram()
    for i in histogram:
        file.write(str(i) + " ")

    file.write("\nMean length of prefixes: " + str(prefixset.get_mean_length()))

    my_trie = lctrie.LCTrie()
    my_trie.load_prefixset(prefixset)

    report = my_trie.report_memory()
    file.write("\nPrefixes: " + str(report['prefixes']))
    file.write("\nNodes: " + str(report['nodes']))
    file.write("\nMemory accesses in worst case: " + str(report['depth']))
    file.write("\nMemory demands [Bytes]\n")
    file.write("32-bit pointer/number: " + str((report['internal_nodes'] + report['edge_nodes']) *3.0*32/8))
    # compute minimal possible pointer/number size
    prefix_ptr = maskedint.log2(report['prefixes']) # bits needed for pointers to prefixes
    node_ptr = maskedint.log2(report['nodes'])   # bits needed for pointers to nodes
    ptr = max(node_ptr, prefix_ptr)   # we need enough bits to represent node_ptr or prefix_ptr
    skip_value = maskedint.log2(report['max_skip'])   # bits needed to represent the skip value
    branch_value = maskedint.log2(report['max_branch'])  # bits needed to represent the branch value
    file.write("\nSmallest possible: " + str((report['nodes']*(ptr + skip_value + branch_value + 0.0))/8))
    file.write("\nPrefix table length: " + str(report['prefix_table_len']))
    file.write("\n" + 80*"*" + "\n")

# try to open a file
try:
    file = open(outfile, 'w')
except IOError:
    print "Cannot open file for writing results"
    exit()

        
print "COMPUTING..."
file.write("\n\n\n==================================================\n")
file.write("----------=======  TEST PREFIXES  =======---------\n")
file.write("==================================================\n")

for prefixset in prefixsets:
    filename = path + prefixset
    parser = genericparser.GenericParser()
    prefixset = parser.get_prefixes()
    if (parser.load_file(filename)):
        print "test file:", filename
        compute()
    else:
        print "Cannot load file",filename
print "DONE.."
print "Results are in file", outfile
file.close()

