###############################################################################
#  test_multimatch.py: Test module for multimatch LPM method
#  Copyright (C) 2010 Brno University of Technology, ANT @ FIT
#  Author(s): Martin Skacan <xskaca00@stud.fit.vutbr.cz>
###############################################################################
#
#  LICENSE TERMS
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#  3. All advertising materials mentioning features or use of this software
#     or firmware must display the following acknowledgement:
#
#       This product includes software developed by the University of
#       Technology, Faculty of Information Technology, Brno and its
#       contributors.
#
#  4. Neither the name of the Company nor the names of its contributors
#     may be used to endorse or promote products derived from this
#     software without specific prior written permission.
#
#  This software or firmware is provided ``as is'', and any express or implied
#  warranties, including, but not limited to, the implied warranties of
#  merchantability and fitness for a particular purpose are disclaimed.
#  In no event shall the company or contributors be liable for any
#  direct, indirect, incidental, special, exemplary, or consequential
#  damages (including, but not limited to, procurement of substitute
#  goods or services; loss of use, data, or profits; or business
#  interruption) however caused and on any theory of liability, whether
#  in contract, strict liability, or tort (including negligence or
#  otherwise) arising in any way out of the use of this software, even
#  if advised of the possibility of such damage.
#
#  $Id$

"""
Test module for multimatch LPM method
"""

import sys, os
import netbench.lpm.parsers.genericparser as genericparser
import netbench.lpm.algorithms.multimatch as multimatch
from netbench.classification import maskedint

path = "../prefixes/test/"
prefixsets = (os.listdir(path))
outfile = "test_multimatch.out"

# compute and print memory demands and stats of currently loaded prefixset
def compute():
    file.write("\n*************** Filename: " + filename + " ***************\n")
    file.write("Count of successful parsed prefixes: " + str(parser.get_parsed_prefixes()))
    file.write("\nCount of invalid prefixes: " + str(parser.get_invalid_prefixes()))
    file.write("\nHistogram:\n\t")
    # get histogram of currently loaded prefixset
    histogram = prefixset.get_histogram()
    for i in histogram:
        file.write(str(i) + " ")

    file.write("\nMean length of prefixes: " + str(prefixset.get_mean_length()))
    
    file.write("\n --- MULTI-MATCH ---")
    # parameters which will be applied for multimatch
    parameters = [4,6]
    for p in parameters:
        file.write("\n" + 40*"-")
        file.write("\nNORMAL " + str(p))
        # uniform distribution in prefix classes
        my_multi = multimatch.MultiMatch(p,1)
        my_multi.load_prefixset(prefixset)
        report = my_multi.report_memory()
        file.write("\nPrefix length | Table size | Prefixes in it")
        file.write("\n-------------------------------------------")
        sum = 0
        sum_prefixes = 0
        for i in range(0,len(report['prefixes'])):
            str1 = str(my_multi.expand_mask[i])
            str2 = str(report['tables'][i])
            # sum of the table sizes
            sum += report['tables'][i]
            # sum of the prefixes in tables
            sum_prefixes += report['prefixes'][i]
            file.write('\n')
            file.write(str1.center(14, ' ') + "|" + str2.center(12, ' ') + "|  " + str(report['prefixes'][i]))
        file.write("\n-------------------------------------------\n")
        file.write("SUM".center(14, ' ') + "|" + str(sum).center(12, ' ') + "|  " + str(sum_prefixes))
        file.write("\nMemory demands [Bytes]")
        file.write("\n32-bit pointer: " + str(sum*32/8.0))
        file.write("\nSmallest possible: " + str(sum*maskedint.log2(parser.get_parsed_prefixes())/8.0))

        file.write("\n" + 40*"-")
        """
        Probably not working with IPv6..?
        
        file.write("\nOPTIMAL " + str(p))
        # best possible distribution in prefix classes
        my_multi = multimatch.MultiMatch(p,2)
        my_multi.load_prefixset(prefixset)
        report = my_multi.report_memory()
        file.write("\nPrefix length | Table size | Prefixes in it")
        file.write("\n-------------------------------------------")
        sum = 0
        sum_prefixes = 0
        for i in range(0,len(report['prefixes'])):
            str1 = str(my_multi.expand_mask[i])
            str2 = str(report['tables'][i])
            # sum of the table sizes
            sum += report['tables'][i]
            # sum of the prefixes in tables
            sum_prefixes += report['prefixes'][i]
            file.write('\n')
            file.write(str1.center(14, ' ') + "|" + str2.center(12, ' ') + "|  " + str(report['prefixes'][i]))
        file.write("\n-------------------------------------------\n")
        file.write("SUM".center(14, ' ') + "|" + str(sum).center(12, ' ') + "|  " + str(sum_prefixes))
        file.write("\nMemory demands [Bytes]")
        file.write("\n32-bit pointer: " + str(sum*32/8.0))
        file.write("\nSmallest possible: " + str(sum*maskedint.log2(parser.get_parsed_prefixes())/8.0))
        """
    file.write("\n" + 80*"*" + "\n")

# try to open a file
try:
    file = open(outfile, 'w')
except IOError:
    print "Cannot open file for writing results"
    exit()

        
print "COMPUTING..."
file.write("\n\n\n==================================================\n")
file.write("----------=======  TEST PREFIXES  =======---------\n")
file.write("==================================================\n")

for prefixset in prefixsets:
    filename = path + prefixset
    parser = genericparser.GenericParser()
    prefixset = parser.get_prefixes()
    if (parser.load_file(filename)):
        print "test file:", filename
        compute()
    else:
        print "Cannot load file",filename
print "DONE.."
print "Results are in file", outfile
file.close()
