###############################################################################
#  generic_parser.py: Module for parsing IPv4 prefixes
#  Copyright (C) 2010 Brno University of Technology, ANT @ FIT
#  Author(s): Frantisek Sedlar <xsedla76@stud.fit.vutbr.cz>
###############################################################################
#
#  LICENSE TERMS
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#  3. All advertising materials mentioning features or use of this software
#     or firmware must display the following acknowledgement:
#
#       This product includes software developed by the University of
#       Technology, Faculty of Information Technology, Brno and its
#       contributors.
#
#  4. Neither the name of the Company nor the names of its contributors
#     may be used to endorse or promote products derived from this
#     software without specific prior written permission.
#
#  This software or firmware is provided ``as is'', and any express or implied
#  warranties, including, but not limited to, the implied warranties of
#  merchantability and fitness for a particular purpose are disclaimed.
#  In no event shall the company or contributors be liable for any
#  direct, indirect, incidental, special, exemplary, or consequential
#  damages (including, but not limited to, procurement of substitute
#  goods or services; loss of use, data, or profits; or business
#  interruption) however caused and on any theory of liability, whether
#  in contract, strict liability, or tort (including negligence or
#  otherwise) arising in any way out of the use of this software, even
#  if advised of the possibility of such damage.
#
#  $Id$

"""
Module for parsing prefixes
"""

import sys
import bprefixparser
import ipv4parser
import ipv6parser

class GenericParser(bprefixparser.BPrefixParser):
    """
    Generic prefix parser.
    """
    
    def __init__(self):
        """Constructor"""
        bprefixparser.BPrefixParser.__init__(self)
        self.lines = 0
        self.parsed_prefixes = 0
        self.invalid_prefixes = 0


    def load_file(self, filename):
        """
        Load prefixes from a file, return True if parsing was successfull.

        Tries to parse the file with ipv4parser and ipv6parser and uses output of the parser, which finds more prefixes.
         
        filename: Name of file in appropriate format.
        """
        parser4 = ipv4parser.Ipv4Parser()
        prefixset4 = parser4.get_prefixes()
        
        parser6 = ipv6parser.Ipv6Parser()
        prefixset6 = parser6.get_prefixes()        

        if (parser4.load_file(filename)) == False:
            return False
                   
        if (parser6.load_file(filename) == False):
            return False
        
        if parser4.get_parsed_prefixes() > parser6.get_parsed_prefixes():
            self.prefixset.set_prefixes(parser4.get_prefixes()) 
            self.parser = parser4
        else:
            self.prefixset.set_prefixes(parser6.get_prefixes()) 
            self.parser = parser6           
            
        return True 


    def get_parsed_prefixes(self):
        """ returns count of successful parsed prefixes from last loaded file """
        return self.parser.parsed_prefixes
        
    def get_invalid_prefixes(self):
        """ returns count of invalid prefixes in last loaded file """
        return self.parser.invalid_prefixes
        
    def get_count_of_lines(self):
        """ returns count of lines in last loaded file """
        return self.parser.lines
        
    def get_lines_of_file(self, filename):
        """ returns count of lines in given file """
        try:
            self._file = open(filename, 'r')
        except IOError:
            return -1
            
        count_lines = len(self._file.readlines())
        self._file.close()
        return count_lines
            
        
        