############################################################################
#  extend_fa.py: Module for PATTERN MATCH algorithm Extending Finite Automat
#  Copyright (C) 2010 Brno University of Technology, ANT @ FIT
#  Author(s): Jaroslav Suchodol
############################################################################
#
#  LICENSE TERMS
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#  3. All advertising materials mentioning features or use of this software
#     or firmware must display the following acknowledgement:
#
#       This product includes software developed by the University of
#       Technology, Faculty of Information Technology, Brno and its
#       contributors.
#
#  4. Neither the name of the Company nor the names of its contributors
#     may be used to endorse or promote products derived from this
#     software without specific prior written permission.
#
#  This software or firmware is provided ``as is'', and any express or
#  implied warranties, including, but not limited to, the implied warranties
#  of merchantability and fitness for a particular purpose are disclaimed.
#  In no event shall the company or contributors be liable for any
#  direct, indirect, incidental, special, exemplary, or consequential
#  damages (including, but not limited to, procurement of substitute
#  goods or services; loss of use, data, or profits; or business
#  interruption) however caused and on any theory of liability, whether
#  in contract, strict liability, or tort (including negligence or
#  otherwise) arising in any way out of the use of this software, even
#  if advised of the possibility of such damage.
#
#  $Id$

"""Module for pattern match: algorithm for Extending Finite Automaton."""

# Based on: "Extending Finite Automata to Efficiently Match Perl-Compatible
# Regular Expressions", ISBN: 978-1-60558-210-8
# URL: http://portal.acm.org/citation.cfm?id=1544037

from netbench.pattern_match import sym_char
from netbench.pattern_match import pcre_parser
from netbench.pattern_match.b_nfa import b_nfa
from netbench.pattern_match import aux_func
import os
import re
import math

class extend_fa(b_nfa):
    """
        Class for Extending Finite Automaton.
    """

    def __init__(self, file_name):
        """
            Construct basic items and make automaton.

            :param file_name: Name of input file
        """

        b_nfa.__init__(self)
        self.make_ext_fa(file_name)

    def __init__(self):
        """
            Construct basic items.
        """
        b_nfa.__init__(self)
        
    def compute(self, file_name):
        """
            Fuction for make Extend FA from RE in file_name.

            :param file_name: Name of input file
            :type file_name: string
        """
        self.make_ext_fa(file_name)
        self._compute = True
    
    def make_ext_fa(self, file_name):
        """
            Fuction for make Extended FA from RE in file_name.

            :param file_name: Name of input file
        """

        # Parse input file.
        parser = pcre_parser.pcre_parser()
        parser.load_file(file_name)
        # Make NFA automata from RE which was in input file.
        self.nfa = b_nfa()
        os.chdir("../../../")
        self.nfa.create_by_parser(parser)
        os.chdir("algorithms/experimental/extend_fa/")
        self.nfa.remove_epsilons()
        # Begin extend algorithm.
        self.remove_excessively_states(file_name)
        self.discover_before_part(file_name)
        self.discover_start_state_counting()
        self.supercede_counting_states()
        self.change_state_numbering()
        self.nfa.get_automaton(False).Flags["Extend_FA"] = True


    def remove_excessively_states(self, file_name):
        """
            Remove states which are excessively (because of queer
            creation NFA).

            :param file_name: Name of input file
        """

        a = self.nfa.get_automaton(False)
        d = []  # states for delete (excessively states)

        self.sort_transitions()
#   # FIRST discover excessively states according start state
#   for t in self.t_s[a.start]:
#     d.append(t[2])
#   d = sorted(d)
#   for s in list(d):
#     if (s + 1) not in d:
#       d.remove(s)
#   # remove excessively states according start state
#   for s in d:
#     # remove from states
#     del a.states[s]
#     # remove from final states
#     if s in a.final:
#       a.final.remove(s)
#     # supercede transition
#     for t in self.t_s[s]:
#       if t[2] == s:
#         # add new tran.
#         a.transitions.add((a.start, t[1], a.start))
#       # remove tran.
#       a.transitions.remove(t)
#   # remove tran. from start state to deleted state
#   for t in self.t_s[a.start]:
#     if t[2] in d:
#       a.transitions.remove(t)
        # SECOND discover excessively states for all other states
        d = []
        for s in a.states:
            if s != 0 \
            and s not in d \
            and len(self.t_s[s]) == 2 \
            and a.states.has_key(s + 1) \
            and len(self.t_s[s + 1]) == 2 \
            and (self.t_s[s+1][0][2] == (s+1) or self.t_s[s+1][1][2] == (s+1)):
                d.append(s + 1)
        # remove excessively states
        for s in d:
            # remove from states
            del a.states[s]
            # remove from final states
            if s in a.final:
                a.final.remove(s)
            # supercede transition
            for t in self.t_s[s - 1]:
                if t[2] == s:
                    # remove tran.
                    a.transitions.remove(t)
                    # add new tran.
                    a.transitions.add((s - 1, t[1], s - 1))
            # remove tran. for state s
            for t in self.t_s[s]:
                a.transitions.remove(t)


    def discover_before_part(self, file_name):
        """
            Discover string which is before .{n} like pattern.

            :param file_name: Name of input file
        """

        self.b = []   # before part of .{n} like pattern
        self.ctr = [] # counters

        fr = open(file_name, 'r')
        for line in fr.readlines():
            # remove '\n' from end of line
            line = line.rsplit('\n', 1)[0]
            pattern = re.compile(r"""
            [.](?=[{])      # like .{15}
            (?![^[]*?\])
            |
            .(?=[{])        # like a{15}
            (?![^[]*?\])
            |
            \[\^?           # like [abc]{15} or [^abc]{15}
            .*?
            (?<!\\)\](?=[{])
            (?![^[]*?\])
            """, re.X)
            # split line to before (split[0]) and after (split[1]) part
            split = re.split(pattern, line, maxsplit=1)
            # remove .* from begin of pattern
            if split[0].find(".*") != -1:
                split[0] = '/' + split[0][split[0].find(".*") + 2:]
            # line contain .{n} like pattern
            if len(split) == 2:
                self.b.append(split[0][1:])
                self.ctr.append(int(split[1][1:split[1].find('}', 1)]))
        fr.close()


    def discover_start_state_counting(self):
        """
            Discover first NFA state from which is begin counting.
        """

        a = self.nfa.get_automaton(False)
        c_s = a.start # current state
        self.s_s_c = [] # state start counting

        self.sort_transitions()
        for key in self.b:
            for c in key:
                for t in self.t_s[c_s]:
                    # TODO: lepsi nalezeni stavu (lepsi algoritmus funkce)
                    if isinstance(a.alphabet[t[1]], sym_char.b_Sym_char):
                        if c == a.alphabet[t[1]].char:
                            # move to destinaton state
                            c_s = t[2]
            self.s_s_c.append(c_s)
            c_s = a.start


    def supercede_counting_states(self):
        """
            Supercede N counting states by one state with special ability.
        """

        a = self.nfa.get_automaton(False)
        d = []  # deleted states
        # FLAGS: 0 = allocation, 1 = hold in this state, 2 = de-allocation

        for i in range(0, len(self.ctr)):
            ctr = self.ctr[i]   # counter
            s_c = self.s_s_c[i] # start counting
            # discover states for delete
            for s in range(s_c + 2, s_c + ctr + 1):
                d.append(s)
            # discover cnt != N tran.
            if (ctr > 1):
                for t in self.t_s[s_c]:
                    if t[2] == (s_c + 1):
                        a.transitions.add((t[2], t[1], t[2], 1, i))
                        break
            # discover allocation tran.
            for t in self.t_s[s_c]:
                if t[2] == (s_c + 1):
                    a.transitions.remove(t)
                    a.transitions.add((t[0], t[1], t[2], 0, i))
            # discover de-allocation tran.
            for t in self.t_s[s_c + ctr]:
                if t[2] == (s_c + ctr + 1):
                    a.transitions.add((s_c + 1, t[1], t[2], 2, i))
            # remove tran. from s_c + 1 to s_c + 2,
            # because of removing state s_c + 2
            for t in self.t_s[s_c + 1]:
                if t[2] == (s_c + 2):
                    a.transitions.remove((t))
        # remove states marked for delete
        for s in d:
            # remove from states
            del a.states[s]
            # remove from final states
            if s in a.final:
                a.final.remove(s)
            # remove transitions
            for t in self.t_s[s]:
                a.transitions.remove((t))


    def change_state_numbering(self):
        """
            Change state numbering back to linear (0, 1, 2, 3, 4, 5, ...).
            Because of removing some states and for better work in C algorithm.
        """

        a = self.nfa.get_automaton(False)
        s_a = sorted(a.states.keys()) # states auxiliary

        # change states
        aux = {}  # auxiliary
        for s in a.states:
            aux[s_a.index(s)] = a.states[s]
            aux[s_a.index(s)]._id = s_a.index(s)
        a.states = aux
        # change final states
        aux = set()
        for s in a.final:
            aux.add(s_a.index(s))
        a.final = aux
        # change transitions
        aux = set()
        for t in a.transitions:
            if len(t) == 3:
                aux.add((s_a.index(t[0]), t[1], s_a.index(t[2])))
            else :
                aux.add((s_a.index(t[0]), t[1], s_a.index(t[2]), t[3:]))
        a.transitions = aux


    def sort_transitions(self):
        """
            Make dictionary of sorted transitions, key is number of state.
        """

        a = self.nfa.get_automaton(False)
        self.t_s = {} # transitions sorted

        for s in a.states.keys():
            self.t_s[s] = []
        for t in a.transitions:
            self.t_s[t[0]].append(t)


    def SaveToFile(self, file_name):
        """
            Make file which represent the Extend automaton.
            This file will be input into algorithm written in C language.

            :param file_name: Name of output file

            NOTE: This method is deprecated. Use save_to_file().
        """
        aux_func.deprecation_warning("method", "SaveToFile()", "save_to_file()")
        self.save_to_file(file_name)

    def save_to_file(self, file_name):
        """
            Make file which represent the Extend automaton.
            This file will be input into algorithm written in C language.

            :param file_name: Name of output file
        """

        a = self.nfa.get_automaton(False)

        fw = open(file_name, 'w')
        # write COUNT OF STATES
        fw.write(str(len(a.states)) + '\n')
        # write START STATE
        fw.write(str(a.start) + '\n')
        # write TRANSITIONS
        self.sort_transitions()
        for s in a.states:
            fw.write("t_c: " + str(len(self.t_s[s])) + '\n')
            for t in self.t_s[s]:
                if isinstance(a.alphabet[t[1]], sym_char.b_Sym_char):
                    fw.write("c_c: 1, d: " + str(t[2]) + '\n')
                    fw.write(str(ord(a.alphabet[t[1]].char)) + "|\n")
                else :
                    fw.write("c_c: " + \
                    str(len(a.alphabet[t[1]].charClass)) + \
                    ", d: " + str(t[2]) + '\n')
                    chars = ""
                    for c in a.alphabet[t[1]].charClass:
                        chars += str(ord(c)) + '|'
                    fw.write(chars + '\n')
                # flag and ctr for transition
                if len(t) == 3:
                    # flag 5 mean not set (this tran. have not flag)
                    fw.write("f: 5, ctr: 0\n")
                else :
                    fw.write("f: " + str(t[3][0]) + ", ctr: " + str(t[3][1]) \
                    + "\n")
        # write FINAL STATES
        final = ""
        for s in a.final:
            final += str(s) + '|'
        fw.write(str(len(a.final)) + '\n' + final + '\n')
        # write COUNTERS
        fw.write("ctr_c: " + str(len(self.ctr)) + '\n')
        ctr_s = ""
        for ctr in self.ctr:
            ctr_s += str(ctr) + '|'
        fw.write(ctr_s + '\n')


    def show(self, file_name):
        """
            Print states, alphabet, start, transitions, final, flags of automat.
            And save graphviz dot file, representing graphical structure of
            automat.

            :param file_name: Name of output file
        """

        print '*' * 80
        print "STATES:", self.nfa.get_automaton(False).states, '\n'
        print "ALPHABET:", self.nfa.get_automaton(False).alphabet, '\n'
        print "START STATE:", self.nfa.get_automaton(False).start, '\n'
        print "TRANSITIONS:", self.nfa.get_automaton(False).transitions, '\n'
        print "FINAL STATES:", self.nfa.get_automaton(False).final, '\n'
        print "FLAGS OF AUTOMAT:", self.nfa.get_automaton(False).Flags
        print '*' * 80
        self.nfa.get_automaton(False).show(file_name)

    def get_state_num(self):
        """
            Return number of states in Extend automaton.

            :returns: Number of states in Extend automaton
            :rtype: int
        """

        return len(self.nfa.get_automaton(False).states)

    def get_trans_num(self):
        """
            Return number of transitions in Extend automaton.

            :returns: Number of transitions in Extend automaton
            :rtype: int
        """

        return len(self.nfa.get_automaton(False).transitions)
        
    def report_memory_optimal(self):
        """
            Report consumed memory in bytes. Optimal mapping algorithm is used \
            (with oracle). Basic algorithm for this variant of mapping is:     \
            M = \|transitions\| * ceil(log(\|states\|, 2) / 8).
            For Extend is: basic + Mem(cnt) + (Tran*2/8).
            
            :returns: Returns number of bytes.
            :rtype: int
        """
        tr_len = len(self.nfa._automaton.transitions)
        st_len = len(self.nfa._automaton.states)
        return int(tr_len * math.ceil(math.log(st_len, 2) / 8)) + \
            (len(self.ctr) * 16 / 8) + (tr_len*2/8);
    
    def report_memory_naive(self):
        """
            Report consumed memory in bytes. Naive mapping algorithm is used \
            (2D array). Basic algorithm for this variant of mapping is:      \
            M = \|states\| * \|alphabet\| * ceil(log(\|states\| + 1, 2) / 8)
            For Extend is: basic + Mem(cnt) + (State * Sym * 2/8)
            
            :returns: Returns number of bytes.
            :rtype: int
        """
        st_len = len(self.nfa._automaton.states)
        al_len = len(self.nfa._automaton.alphabet)
        return int(st_len * al_len * math.ceil(math.log(st_len + 1, 2) / 8)) +\
            (len(self.ctr) * 16 / 8) + (st_len*al_len*2/8);