###############################################################################
#  test_extend_fa.py: Test module for class extend_fa
#  Copyright (C) 2012 Brno University of Technology, ANT @ FIT
#  Author(s): Vlastimil Kosar, <ikosar@fit.vutbr.cz>
###############################################################################
#
#  LICENSE TERMS
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#  3. All advertising materials mentioning features or use of this software
#     or firmware must display the following acknowledgement:
#
#       This product includes software developed by the University of
#       Technology, Faculty of Information Technology, Brno and its
#       contributors.
#
#  4. Neither the name of the Company nor the names of its contributors
#     may be used to endorse or promote products derived from this
#     software without specific prior written permission.
#
#  This software or firmware is provided ``as is, and any express or implied
#  warranties, including, but not limited to, the implied warranties of
#  merchantability and fitness for a particular purpose are disclaimed.
#  In no event shall the company or contributors be liable for any
#  direct, indirect, incidental, special, exemplary, or consequential
#  damages (including, but not limited to, procurement of substitute
#  goods or services; loss of use, data, or profits; or business
#  interruption) however caused and on any theory of liability, whether
#  in contract, strict liability, or tort (including negligence or
#  otherwise) arising in any way out of the use of this software, even
#  if advised of the possibility of such damage.
#
#  $Id$

from netbench.pattern_match.algorithms.experimental.extend_fa.extend_fa import extend_fa
from netbench.pattern_match.parser import parser
from netbench.pattern_match.nfa_data import nfa_data
from netbench.pattern_match.b_state import b_State
from netbench.pattern_match.sym_char import b_Sym_char
from netbench.pattern_match.sym_char_class import b_Sym_char_class
from netbench.pattern_match import aux_func
import unittest, copy
import pickle
import os

class test_extend_fa(unittest.TestCase):
    """
        Test module for class extend_fa.
    """
    def test_compute(self):
        """compute()"""
        # Check the correctness of the logical machine output over
        # self.assertTrue on individual items + focus on the properties
        # of Extend FA

        # /abcd/; test with an expression that does not use properties
        # of Extend FA
        ext_fa = extend_fa()
        ext_fa.compute(aux_func.getPatternMatchDir() + "/algorithms/experimental/extend_fa/test_data/pattern0.re")
        
        f = open(aux_func.getPatternMatchDir() + "/algorithms/experimental/extend_fa/test_data/test_data_0.nfa_data", "r")
        result = pickle.load(f)
        f.close()
        
        self.assertTrue(sorted(ext_fa._automaton.states.keys()) ==
            sorted(result._automaton.states.keys()))
        self.assertTrue(ext_fa._automaton.alphabet == result._automaton.alphabet)
        self.assertTrue(ext_fa._automaton.start == result._automaton.start)
        self.assertTrue(ext_fa._automaton.final == result._automaton.final)
        self.assertTrue(ext_fa._automaton.transitions == result._automaton.transitions)
        self.assertTrue(ext_fa.ctr == result.ctr)
        
        # /ab.*cd /; test with an expression that contains .*
        ext_fa = extend_fa()
        ext_fa.compute(aux_func.getPatternMatchDir() + "/algorithms/experimental/extend_fa/test_data/pattern1.re")
        
        f = open(aux_func.getPatternMatchDir() + "/algorithms/experimental/extend_fa/test_data/test_data_1.nfa_data", "r")
        result = pickle.load(f)
        f.close()

        self.assertTrue(sorted(ext_fa._automaton.states.keys()) ==
            sorted(result._automaton.states.keys()))
        self.assertTrue(ext_fa._automaton.alphabet == result._automaton.alphabet)
        self.assertTrue(ext_fa._automaton.start == result._automaton.start)
        self.assertTrue(ext_fa._automaton.final == result._automaton.final)
        self.assertTrue(ext_fa._automaton.transitions == result._automaton.transitions)
        self.assertTrue(ext_fa._automaton.Flags == result._automaton.Flags)
        self.assertTrue(ext_fa.ctr == result.ctr)
        
        # /ab[^1234]*cd|efg/; test with an expression containing one
        # alternation [^1234]*, the second is not
        ext_fa = extend_fa()
        ext_fa.compute(aux_func.getPatternMatchDir() + "/algorithms/experimental/extend_fa/test_data/pattern2.re")
        
        f = open(aux_func.getPatternMatchDir() + "/algorithms/experimental/extend_fa/test_data/test_data_2.nfa_data", "r")
        result = pickle.load(f)
        f.close()

        self.assertTrue(sorted(ext_fa._automaton.states.keys()) ==
            sorted(result._automaton.states.keys()))
        self.assertTrue(ext_fa._automaton.alphabet == result._automaton.alphabet)
        self.assertTrue(ext_fa._automaton.start == result._automaton.start)
        self.assertTrue(ext_fa._automaton.final == result._automaton.final)
        self.assertTrue(ext_fa._automaton.transitions == result._automaton.transitions)
        self.assertTrue(ext_fa._automaton.Flags == result._automaton.Flags)
        self.assertTrue(ext_fa.ctr == result.ctr)

    def test_save_to_file(self):
        """save_to_file()"""
        
        ext_fa = extend_fa()
        ext_fa.compute(aux_func.getPatternMatchDir() + "/algorithms/experimental/extend_fa/test_data/pattern2.re")
        ext_fa.save_to_file(aux_func.getPatternMatchDir() + "/algorithms/experimental/extend_fa/test_data/save.dat")
        self.assertTrue(os.path.exists(aux_func.getPatternMatchDir() + "/algorithms/experimental/extend_fa/test_data/save.dat") == True)
        f = open(aux_func.getPatternMatchDir() + "/algorithms/experimental/extend_fa/test_data/save.tmpl","r");
        tmpl_content = f.read()
        f.close()
        f = open(aux_func.getPatternMatchDir() + "/algorithms/experimental/extend_fa/test_data/save.dat","r");
        save_content = f.read()
        f.close()
        self.assertTrue(tmpl_content == save_content)
        aux_func.getstatusoutput("rm -f " + aux_func.getPatternMatchDir() + "/algorithms/experimental/extend_fa/test_data/save.dat", None)
        
    def test_report_memory_optimal(self):
        """report_memory_optimal()"""

        ext_fa = extend_fa()
        ext_fa.compute(aux_func.getPatternMatchDir() + "/algorithms/experimental/extend_fa/test_data/pattern2.re")

        self.assertTrue(ext_fa.report_memory_optimal() == 13)

    def test_report_memory_naive(self):
        """report_memory_naive()"""

        ext_fa = extend_fa()
        ext_fa.compute(aux_func.getPatternMatchDir() + "/algorithms/experimental/extend_fa/test_data/pattern2.re")

        self.assertTrue(ext_fa.report_memory_naive() == 101)

if __name__ == '__main__':
    suite = unittest.TestLoader().loadTestsFromTestCase(test_extend_fa)
    unittest.TextTestRunner(verbosity=2).run(suite)