###############################################################################
#  example_of_use.py: Module for PATTERN MATCH - example of use for Clark's NFA
#  Copyright (C) 2011 Brno University of Technology, ANT @ FIT
#  Author(s): Vlastimil Kosar <ikosar@fit.vutbr.cz>
###############################################################################
#
#  LICENSE TERMS
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#  3. All advertising materials mentioning features or use of this software
#     or firmware must display the following acknowledgement:
#
#       This product includes software developed by the University of
#       Technology, Faculty of Information Technology, Brno and its
#       contributors.
#
#  4. Neither the name of the Company nor the names of its contributors
#     may be used to endorse or promote products derived from this
#     software without specific prior written permission.
#
#  This software or firmware is provided ``as is'', and any express or implied
#  warranties, including, but not limited to, the implied warranties of
#  merchantability and fitness for a particular purpose are disclaimed.
#  In no event shall the company or contributors be liable for any
#  direct, indirect, incidental, special, exemplary, or consequential
#  damages (including, but not limited to, procurement of substitute
#  goods or services; loss of use, data, or profits; or business
#  interruption) however caused and on any theory of liability, whether
#  in contract, strict liability, or tort (including negligence or
#  otherwise) arising in any way out of the use of this software, even
#  if advised of the possibility of such damage.
#
#  $Id$

from netbench.pattern_match import parser
from netbench.pattern_match.algorithms.experimental.nfa_split import nfa_split

"""
    This module demonstrate usage of nfa_split class.
"""
ruleset = "../../../rules/Snort/worm.reg"
#ruleset = "../../../rules/Snort/dos.rules.pcre"

def save_file(name, content):
    """
        Save the content stored in parameter content into file named by \
        parameter name.
        
        :param name: Name of the file.
        :type name: string
        
        :param content: Content to write.
        :type content: string
    """
    # Open file
    f = open(name, "w")
    # Write the content into the file
    f.write(content)
    # Close file
    f.close()

if __name__ == '__main__':
    print("-------------------------------------------------------------------")
    print("                   Example of use: NFA split                       ")
    print("-------------------------------------------------------------------")
    print(" Ruleset: " + ruleset)
    print(" Concurency detection: determinization O(2^n)                      ")
    print(" Genereated VHDL output: det.decoder0.vhd                          ")
    print("                         det.det_part0.vhd                         ")
    print("                         det.encoder0.vhd                          ")
    print("                         det.memory0.vhd                           ")
    print("                         det.nfa_split.vhd                         ")
    print("                         det.nondet_part.vhd                       ")
    print("-------------------------------------------------------------------")
    # Create nfa_split object
    ns = nfa_split.nfa_split()
    # Create parser - use default parser
    Test0 = parser.parser("pcre_parser")
    # Load RE file
    #Test0.load_file("../../../rules/L7/selected.pcre")
    Test0.load_file(ruleset)
    # Parse RE and create NFA
    ns.create_by_parser(Test0)
    # Call the compute method
    ns.create_det_parts(concurrency_method=0)
    # Get number of used LUTs and FFs
    data = ns.report_logic()
    print(" Used LUTs estimation: " + str(data[0]))
    print(" Used FFs estimation: " + str(data[1]))
    print(" Used BRAMs estimation: " + str(data[2]))
    print("-------------------------------------------------------------------")
    # Save implementation
    res = ns.get_HDL()
    for unit in res:
        # Save file
        save_file("det." + unit, res[unit]);
    
    print("-------------------------------------------------------------------")
    print("                   Example of use: NFA split                       ")
    print("-------------------------------------------------------------------")
    print(" Ruleset: " + ruleset)
    print(" Concurency detection: concurrently_reachable O(n^2)               ")
    print(" Genereated VHDL output: cr.decoder0.vhd                           ")
    print("                         cr.det_part0.vhd                          ")
    print("                         cr.encoder0.vhd                           ")
    print("                         cr.memory0.vhd                            ")
    print("                         cr.nfa_split.vhd                          ")
    print("                         cr.nondet_part.vhd                        ")
    print("-------------------------------------------------------------------")
    # Create nfa_split object
    ns = nfa_split.nfa_split()
    # Create parser - use default parser
    Test0 = parser.parser("pcre_parser")
    # Load RE file
    #Test0.load_file("../../../rules/L7/selected.pcre")
    Test0.load_file(ruleset)
    # Parse RE and create NFA
    ns.create_by_parser(Test0)
    # Call the compute method
    ns.create_det_parts(concurrency_method=1)
    # Get number of used LUTs and FFs
    data = ns.report_logic()
    print(" Used LUTs estimation: " + str(data[0]))
    print(" Used FFs estimation: " + str(data[1]))
    print(" Used BRAMs estimation: " + str(data[2]))
    print("-------------------------------------------------------------------")
    # Save implementation
    res = ns.get_HDL()
    for unit in res:
        # Save file
        save_file("cr." + unit, res[unit]);
    
