library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_textio.all;
use std.textio.all;

-- pragma translate_off
library unisim;
use unisim.vcomponents.all;
-- pragma translate_on

-- ----------------------------------------------------------------------------
--                        Entity declaration
-- ----------------------------------------------------------------------------
entity DFSM_MEMORY%$% is
   generic(
      MEMORY_TARGET_WIDTH : integer := %$%;
      MEMORY_SYMBOL_WIDTH : integer := %$%;
      MEMORY_ADDR_WIDTH   : integer := %$%;
      MEMORY_ITEMS        : integer := %$%
   );
   port(
      -- common signals
      CLK            : in  std_logic;
      RESET          : in  std_logic;
      -- common memory interface
      MEMORY_ADDR    : in  std_logic_vector(MEMORY_ADDR_WIDTH - 1 downto 0);
      MEMORY_RQ      : in  std_logic;
      
      -- memory write interface - usualy not conected
      MEMORY_WE      : in std_logic;
      MEMORY_DI      : in std_logic_vector((MEMORY_TARGET_WIDTH + MEMORY_SYMBOL_WIDTH + 3) - 1 downto 0);
      
      -- memory read interface
      TARGET         : out std_logic_vector(MEMORY_TARGET_WIDTH - 1 downto 0);
      SYMBOL         : out std_logic_vector(MEMORY_SYMBOL_WIDTH - 1 downto 0);
      N              : out std_logic;
      F              : out std_logic;
      V              : out std_logic
      
   );
end entity DFSM_MEMORY%$%;

architecture full of DFSM_MEMORY%$% is
    attribute ram_style   : string;

    type t_mem is array(0 to MEMORY_ITEMS - 1) of bit_vector((MEMORY_TARGET_WIDTH + MEMORY_SYMBOL_WIDTH + 3) - 1 downto 0);

    impure function ram_init (ram_file_name : in string) return t_mem is                                                   
       FILE ram_file      : text is in ram_file_name;                       
       variable file_line : line;                                 
       variable init      : t_mem;                                      
    begin                                                        
       for I in t_mem'range loop                                  
           readline (ram_file, file_line);                             
           read (file_line, init(I));                                  
       end loop;                                                    
       return init;                                                  
    end function;
    
    signal memory : t_mem := ram_init("%$%");
    
    signal data_out : std_logic_vector((MEMORY_TARGET_WIDTH + MEMORY_SYMBOL_WIDTH + 3) - 1 downto 0);
begin

   process(CLK)
   begin
      if (CLK'event and CLK = '1') then
         if (MEMORY_RQ = '1') then
            if (MEMORY_WE = '1') then
               memory(conv_integer(unsigned(MEMORY_ADDR))) <= to_bitvector(MEMORY_DI);
            end if;
            data_out <= to_stdlogicvector(memory(conv_integer(unsigned(MEMORY_ADDR))));
         end if;
      end if;
   end process;

   TARGET <= data_out((MEMORY_TARGET_WIDTH + MEMORY_SYMBOL_WIDTH + 3) - 1 downto (MEMORY_SYMBOL_WIDTH + 3));
   SYMBOL <= data_out((MEMORY_SYMBOL_WIDTH + 3) - 1 downto 3);
   N      <= data_out(1);
   F      <= data_out(2);
   V      <= data_out(0);
   
end architecture full;