--
-- TESTBENCH.vhd: fl_sim testbench
-- Copyright (C) 2006 CESNET
-- Author(s): Vlastimil Kosar <xkosar02@stud.fit.vutbr.cz>
--
-- Redistribution and use in source and binary forms, with or without
-- modification, are permitted provided that the following conditions
-- are met:
-- 1. Redistributions of source code must retain the above copyright
--    notice, this list of conditions and the following disclaimer.
-- 2. Redistributions in binary form must reproduce the above copyright
--    notice, this list of conditions and the following disclaimer in
--    the documentation and/or other materials provided with the
--    distribution.
-- 3. Neither the name of the Company nor the names of its contributors
--    may be used to endorse or promote products derived from this
--    software without specific prior written permission.
--
-- This software is provided ``as is'', and any express or implied
-- warranties, including, but not limited to, the implied warranties of
-- merchantability and fitness for a particular purpose are disclaimed.
-- In no event shall the company or contributors be liable for any
-- direct, indirect, incidental, special, exemplary, or consequential
-- damages (including, but not limited to, procurement of substitute
-- goods or services; loss of use, data, or profits; or business
-- interruption) however caused and on any theory of liability, whether
-- in contract, strict liability, or tort (including negligence or
-- otherwise) arising in any way out of the use of this software, even
-- if advised of the possibility of such damage.
--
-- $Id: testbench.vhd 1256 2008-02-10 19:32:02Z xkosar02 $
--
-- TODO:
--
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_textio.all;
use ieee.numeric_std.all;
use std.textio.all;
use work.fl_pkg.all; 
use work.fl_sim_oper.all; 
use work.fl_bfm_pkg.all;
use work.fl_bfm_rdy_pkg.all;

-- library containing log2 function
use work.math_pack.all;
-- ----------------------------------------------------------------------------
--                        Entity declaration
-- ----------------------------------------------------------------------------
entity TESTBENCH is
end entity TESTBENCH;

-- ----------------------------------------------------------------------------
--                      Architecture declaration
-- ----------------------------------------------------------------------------
architecture TESTBENCH_arch of TESTBENCH is


-- -----------------------Testbench constant-----------------------------------
   constant clkper_50       : time := 20 ns;
   constant clkper_100      : time := 10 ns;
   constant reset_time      : time := 100 * clkper_100;
   constant TX_DATA_WIDTH   : integer   := %$%;--32
   constant RX_DATA_WIDTH   : integer   := %$%;--8
   constant RULES           : integer   := %$%;--4
   constant RX_LOG          : string :="";
   constant TX_LOG          : string :="./tests/fl_output.txt";

-- -----------------------Testbench auxilarity signals-------------------------
     -- CLK_GEN Signals
     signal reset             : std_logic;
     signal clk               : std_logic;
     signal clk_50_in         : std_logic;
     signal clk50             : std_logic;
     signal clk100            : std_logic;
     signal lock              : std_logic;
     signal fl_clk            : std_logic;

     -- Frame Link Bus 32 (FL_SIM)
     signal FL_bus_DATA        : std_logic_vector(RX_DATA_WIDTH-1 downto 0);
     signal FL_bus_REM         : std_logic_vector(log2(RX_DATA_WIDTH/8) - 1 downto 0);
     signal FL_bus_SRC_RDY_N   : std_logic;
     signal FL_bus_DST_RDY_N   : std_logic;
     signal FL_bus_SOP_N       : std_logic;
     signal FL_bus_EOP_N       : std_logic;
     signal FL_bus_SOF_N       : std_logic;
     signal FL_bus_EOF_N       : std_logic;
     
     signal FL_bus2_DATA        : std_logic_vector(TX_DATA_WIDTH-1 downto 0);
     signal FL_bus2_REM         : std_logic_vector(log2(TX_DATA_WIDTH/8) - 1 downto 0);
     signal FL_bus2_SRC_RDY_N   : std_logic;
     signal FL_bus2_DST_RDY_N   : std_logic;
     signal FL_bus2_SOP_N       : std_logic;
     signal FL_bus2_EOP_N       : std_logic;
     signal FL_bus2_SOF_N       : std_logic;
     signal FL_bus2_EOF_N       : std_logic;
   
     -- FL_SIM component ctrl      
     signal fl_sim_ctrl        : t_fl_ctrl;
     signal fl_sim_strobe      : std_logic;
     signal fl_sim_busy        : std_logic;
     signal fl_sim_ctrl1        : t_fl_ctrl;
     signal fl_sim_strobe1      : std_logic;
     signal fl_sim_busy1        : std_logic;
     
begin

-- Reset generation -----------------------------------------------------------
   reset_gen : process
   begin
      reset <= '1';
      wait for reset_time;
      reset <= '0';
      wait;
   end process reset_gen;
   
-- clk50 generator ------------------------------------------------------------
clk50_gen : process
begin
   clk_50_in <= '1';
   wait for clkper_50/2;
   clk_50_in <= '0';
   wait for clkper_50/2;
end process;

-- CLK_GEN component ----------------------------------------------------------
CLK_GEN_U: entity work.CLK_GEN
   port map (
      -- Input
      CLK50_IN    => clk_50_in,
      RESET       => '0',
      -- Output
      CLK50_OUT   => clk50,
      CLK25       => open,
      CLK100      => clk100,
      CLK200      => open,
      LOCK        => lock
   );
clk <= clk100;
fl_clk <= clk100;

-- Frame Link Bus simulation component ------------------------------------------
UUT : entity work.WRAPPER
   generic map (
      RX_DATA_WIDTH => RX_DATA_WIDTH,
      TX_DATA_WIDTH => TX_DATA_WIDTH,
      RULES      => RULES
   )
   port map (
    -- Common interface
      RESET           => reset,
      CLK             => fl_clk,

      -- FL Bus Interface
      RX_DATA=>FL_bus_DATA,
      RX_REM=>(others => '0'), --""
      RX_SOF_N=>FL_bus_SOF_N,
      RX_EOF_N=>FL_bus_EOF_N,
      RX_SOP_N=>FL_bus_SOP_N,
      RX_EOP_N=>FL_bus_EOP_N,
      RX_SRC_RDY_N=>FL_bus_SRC_RDY_N,
      RX_DST_RDY_N=>FL_bus_DST_RDY_N,

      TX_DATA=>FL_bus2_DATA,
      TX_REM=>FL_bus2_REM, --open
      TX_SOF_N=>FL_bus2_SOF_N,
      TX_EOF_N=>FL_bus2_EOF_N,
      TX_SOP_N=>FL_bus2_SOP_N,
      TX_EOP_N=>FL_bus2_EOP_N,
      TX_SRC_RDY_N=>FL_bus2_SRC_RDY_N,
      TX_DST_RDY_N=>FL_bus2_DST_RDY_N
     );

MONITOR_I: entity work.MONITOR
   generic map(
      -- FrameLink data bus width
      -- only 8, 16, 32, 64 and 128 bit fl bus supported
      RX_TX_DATA_WIDTH => TX_DATA_WIDTH,
      FILE_NAME        => "./tests/monitor.txt",
      FRAME_PARTS      => 1,
      RDY_DRIVER       => EVER
   )
   port map(
      -- Common interface
      FL_RESET           => reset,
      FL_CLK             => fl_clk,

      -- RX Frame link Interface
      RX_DATA=>FL_bus2_DATA,
      RX_REM=> FL_bus2_REM, -- ""
      RX_SOF_N=>FL_bus2_SOF_N,
      RX_EOF_N=>FL_bus2_EOF_N,
      RX_SOP_N=>FL_bus2_SOP_N,
      RX_EOP_N=>FL_bus2_EOP_N,
      RX_SRC_RDY_N=>FL_bus2_SRC_RDY_N,
      RX_DST_RDY_N=>FL_bus2_DST_RDY_N      
     );
    
     
FL_BFM_U : entity work.FL_BFM
   generic map (
      DATA_WIDTH=>RX_DATA_WIDTH,
      FL_BFM_ID=>0
   )
   port map (
      -- Common interface
      RESET           => reset,
      CLK             => fl_clk,

      TX_DATA=>FL_bus_DATA,
      TX_REM=>open, -- open
      TX_SOF_N=>FL_bus_SOF_N,
      TX_EOF_N=>FL_bus_EOF_N,
      TX_SOP_N=>FL_bus_SOP_N,
      TX_EOP_N=>FL_bus_EOP_N,
      TX_SRC_RDY_N=>FL_bus_SRC_RDY_N,
      TX_DST_RDY_N=>FL_bus_DST_RDY_N
     );

     
tb : process

begin
SetSeed(5958965);
wait for reset_time;
SendWriteFile("./tests/fl_sim.txt", RND, flCmd_0, 0);
wait;
end process;

end architecture TESTBENCH_arch;